import os

from datapackage import Package, Resource
from .. import DataStreamProcessor
from ..helpers.resource_matcher import ResourceMatcher


class load(DataStreamProcessor):

    def __init__(self, load_source, name=None, resources=None, **options):
        super(load, self).__init__()
        self.load_source = load_source
        self.options = options
        self.name = name
        self.resources = resources
        self.load_dp = None

    def process_datapackage(self, dp: Package):
        if isinstance(self.load_source, tuple):
            datapackage_descriptor, _ = self.load_source
            dp.descriptor.setdefault('resources', [])
            self.resource_matcher = ResourceMatcher(self.resources, datapackage_descriptor)
            for resource_descriptor in datapackage_descriptor['resources']:
                if self.resource_matcher.match(resource_descriptor['name']):
                    dp.add_resource(resource_descriptor)
        elif os.path.basename(self.load_source) == 'datapackage.json':
            self.load_dp = Package(self.load_source)
            self.resource_matcher = ResourceMatcher(self.resources, self.load_dp)
            dp.descriptor.setdefault('resources', [])
            for resource in self.load_dp.resources:
                if self.resource_matcher.match(resource.name):
                    dp.add_resource(resource.descriptor)
        else:
            if os.path.exists(self.load_source):
                base_path = os.path.dirname(self.load_source) or '.'
                self.load_source = os.path.basename(self.load_source)
            else:
                base_path = None
            descriptor = dict(path=self.load_source,
                              profile='tabular-data-resource')
            if 'format' in self.options:
                descriptor['format'] = self.options['format']
            self.res = Resource(descriptor,
                                base_path=base_path,
                                **self.options)
            self.res.infer(confidence=1, limit=1000)
            if self.name is not None:
                self.res.descriptor['name'] = self.name
            self.res.descriptor['path'] = '{name}.{format}'.format(**self.res.descriptor)
            dp.add_resource(self.res.descriptor)
        return dp

    def process_resources(self, resources):
        yield from super(load, self).process_resources(resources)
        if isinstance(self.load_source, tuple):
            datapackage_descriptor, resources = self.load_source
            yield from (resource for resource, descriptor in zip(resources, datapackage_descriptor['resources'])
                        if self.resource_matcher.match(descriptor['name']))
        elif self.load_dp is not None:
            yield from (resource.iter(keyed=True) for resource in self.load_dp.resources
                        if self.resource_matcher.match(resource.name))
        else:
            yield self.res.iter(keyed=True)
